# coding=utf-8

"""检查结果返回值"""
CHECK_PASS = 0
CHECK_FAIL = 1

import time
import os
import sys
import re
import traceback
path = os.path.dirname(os.path.abspath(__file__))
sys.path.append(path)
from util import log

CLI = None
LANGUAGE = None
IP_ADDR = None


def execute(context):
    """
    Function name      : getdev_type
    Function describe  : 添加主机获取信息
    Input              : context
    Return             : flag, cliRet, errMsg
    """
    # 给全局变量赋初始值，按照要求传递上下文进去。
    global logger
    logger = context.get("Logger")
    global CLI
    CLI = context.get("SSH")
    global LANGUAGE
    LANGUAGE = "en"
    global IP_ADDR
    IP_ADDR = context.get("dev").getIp()

    # is FusionSphere
    isFusionSphere = CLI.execCmdWithTimout("cat /opt/galax/version/GalaxVersion.cfg", 60)
    log.info(context, isFusionSphere)
    fusionSpherePattern = re.compile(r"V\d{3}R\d{3}C\d{2}RC\S{1,2}|V\d{3}R\d{3}C\d{2}SPC\S{3}|V\d{3}R\d{3}C\d{2}SPH\S{3}|V\d{3}R\d{3}C\d{2}CP\S{4}|V\d{3}R\d{3}C\d{2}HP\S{4}|V\d{3}R\d{3}C\d{2}")
    verGroups = fusionSpherePattern.findall(isFusionSphere)
    if verGroups:
        return {"name":"Not Linux", "type":"Not Found", "sn":"", "version":""}

    name = 'Not Linux'
    dev_type = ''
    sn = ''
    version = 'Not Found'
    name = check_and_get_result(get_device_name())
    if name:
        try:
            sn = check_and_get_result(get_serial_number())
            version = check_and_get_result(get_product_version())
            dev_type = check_and_get_result(get_dev_type())
            logger.info("the device name is: %s, the type is : %s, the sn is : %s, the version is : %s" % (name, dev_type, sn, version))
        except:
            logger.error("the device name is: %s, the type is : %s, the sn is : %s, the version is : %s" % (name, dev_type, sn, version))
            logger.error("the except info is:" + traceback.format_exc())

    return {"name": name, "type":dev_type, "sn": sn, "version": version}

# 取中间一个有效值
def check_and_get_result((fun_flag, result, fun_err_msg)):
    """

    :rtype : object
    """
    return str(result)


def get_dev_type():
    """
    Function name      : getdev_type
    Function describe  : 获取操作系统类型
    Input              : 
    Return             : funFlag, dev_type, funErrMsg
    """
    fun_flag = CHECK_FAIL
    dev_type = ''
    fun_err_msg = ''
    op_sys_info = CLI.execCmdWithTimout("cat /etc/*release", 60)

    if 'No such file or directory' not in op_sys_info:
        tem_line = op_sys_info.splitlines()
        if 'Asianux' in op_sys_info:
            for i in range(0, len(tem_line)):
                #####################     Suse Area    #####################
                if 'Asianux' in tem_line[i]:
                    dev_type = 'Asianux ' + tem_line[i].split()[2]
                    fun_flag = CHECK_PASS
                    break
        elif 'Debian' in op_sys_info:
            for i in range(0, len(tem_line)):
                if 'Debian' in tem_line[i]:
                    dev_type = 'Debian ' + tem_line[i].split()[2]
                    fun_flag = CHECK_PASS
                    break
        elif 'Oracle Linux' in op_sys_info:
            for i in range(0, len(tem_line)):
                if 'Oracle Linux' in tem_line[i]:
                    info_arrays = tem_line[i].split()
                    matchVer = re.findall(r'\d+.\d+',tem_line[i],re.I)
                    version = info_arrays[-1] if len(matchVer) < 1 else matchVer[0]
                    version = info_arrays[4] if len(info_arrays) > 4 else version
                    dev_type = 'Oracle Linux ' + version
                    fun_flag = CHECK_PASS
                    break
        elif 'SUSE' in op_sys_info:
            version = "0"
            patchVersion = "0"
            for i in range(0, len(tem_line)):
                if tem_line[i].startswith("VERSION ="):
                    version = tem_line[i].split("=")[1].strip()
                if tem_line[i].startswith("PATCHLEVEL ="):
                    patchVersion = tem_line[i].split("=")[1].strip()
            if "0" == patchVersion:
                dev_type = 'SUSE %s' % (version)
            else:
                dev_type = 'SUSE %s SP%s' % (version, patchVersion)
                #####################   Red Hat Area    #####################
        elif 'Red Hat' in op_sys_info:
            for i in range(0, len(tem_line)):
                if 'Red Hat' in tem_line[i] and "=" not in tem_line[i]:
                    dev_type = "Red Hat " + tem_line[i].split()[6]
                    fun_flag = CHECK_PASS
                    break
        elif 'CentOS' in op_sys_info:
            for i in range(0, len(tem_line)):
                if 'CentOS' in tem_line[i]:
                    dev_type = "CentOS " + tem_line[i].split()[2]
                    if dev_type == 'CentOS release':
                        dev_type = "CentOS " + tem_line[i].split()[3]
                    fun_flag = CHECK_PASS
                    break
        elif 'XenServer' in op_sys_info:
            for i in range(0, len(tem_line)):
                if 'XenServer' in tem_line[i]:
                    dev_type = "XenServer " + re.split(' |-', tem_line[i])[2];
                    fun_flag = CHECK_PASS
                    break
                #####################   Ubuntu Area    #####################
        elif 'Ubuntu' in op_sys_info:
            for i in range(0, len(tem_line)):
                if 'Ubuntu' in tem_line[i]:
                    dev_type = fetch_ubuntu_base_info()
                    fun_flag = CHECK_PASS
                    break

    if not dev_type:
        op_sys_info = CLI.execCmdWithTimout("cat /etc/issue", 60)
        tem_line = op_sys_info.splitlines()
        if 'Asianux' in op_sys_info:
            for i in range(0, len(tem_line)):
                #####################     Suse Area    #####################
                if 'Asianux' in tem_line[i]:
                    dev_type = 'Asianux ' + tem_line[i].split()[2]
                    fun_flag = CHECK_PASS
                    break
        elif 'Debian' in op_sys_info:
            for i in range(0, len(tem_line)):
                if 'Debian' in tem_line[i]:
                    dev_type = 'Debian ' + tem_line[i].split()[2]
                    fun_flag = CHECK_PASS
                    break
        elif 'Oracle Linux' in op_sys_info:
            for i in range(0, len(tem_line)):
                if 'Oracle Linux' in tem_line[i]:
                    dev_type = 'Oracle Linux ' + tem_line[i].split()[4]
                    fun_flag = CHECK_PASS
                    break
        elif 'SUSE' in op_sys_info:
            for i in range(0, len(tem_line)):
                if 'SUSE' in tem_line[i]:
                    dev_type = "SUSE " + tem_line[i].split()[6] + " " + tem_line[i].split()[7]
                    fun_flag = CHECK_PASS
                    break
                #####################   Red Hat Area    #####################
        elif 'Red Hat' in op_sys_info:
            for i in range(0, len(tem_line)):
                if 'Red Hat' in tem_line[i]:
                    dev_type = "Red Hat " + tem_line[i].split()[6]
                    fun_flag = CHECK_PASS
                    break
        elif 'CentOS' in op_sys_info:
            for i in range(0, len(tem_line)):
                if 'CentOS' in tem_line[i]:
                    dev_type = "CentOS " + tem_line[i].split()[2]
                    if dev_type == 'CentOS release':
                        dev_type = "CentOS " + tem_line[i].split()[3]
                    fun_flag = CHECK_PASS
                    break
        elif 'XenServer' in op_sys_info:
            for i in range(0, len(tem_line)):
                if 'XenServer' in tem_line[i]:
                    dev_type = "XenServer " + re.split(' |-', tem_line[i])[2];
                    fun_flag = CHECK_PASS
                    break
                #####################   Ubuntu Area    #####################
        elif 'Ubuntu' in op_sys_info:
            for i in range(0, len(tem_line)):
                if 'Ubuntu' in tem_line[i]:
                    dev_type = fetch_ubuntu_base_info()
                    fun_flag = CHECK_PASS
                    break

    if dev_type == '':
        dev_type = "Unknown"
    return fun_flag, dev_type, fun_err_msg


# need to explain this function.
def fetch_ubuntu_base_info():
    cli_ret = CLI.execCmdWithTimout("cat /etc/issue", 60)
    cli_ret_list = cli_ret.splitlines()
    cli_ret_lists = cli_ret_list[1].split(" ")
    return cli_ret_lists[0] + cli_ret_lists[1]


# need to explain this function.
def get_device_name():
    """
    Function name      : getdev_type
    Function describe  : 获取主机名
    Input              : 
    Return             : fun_flag, deviceName, fun_err_msg
    """
    fun_flag = CHECK_FAIL
    fun_err_msg = ''
    device_name = ''
    cli_ret = CLI.execCmdWithTimout("uname -a", 60)
    for line in cli_ret.splitlines():
        if line.lower().startswith("linux"):
            device_name = "Linux"
            break
    logger.info("the device name is:" + device_name)

    if '' == device_name:
        if LANGUAGE == "zh":
            fun_err_msg = u"获取主机名称失败。"
        else:
            fun_err_msg = "Get hostname failed."
    else:
        fun_flag = CHECK_PASS
    return fun_flag, device_name, fun_err_msg


# need to explain this function.
def get_product_version():
    """
    Function name      : getdev_type
    Function describe  : 获取操作系统版本号
    Input              : 
    Return             : cliFlag, productVersion, fun_err_msg
    """
    cli_flag = CHECK_FAIL
    product_version = ''
    fun_err_msg = ''
    op_sys_ver = CLI.execCmdWithTimout("cat /proc/version", 60)
    for line in op_sys_ver.splitlines():
        if "linux" in line.lower():
            product_version = re.split(' |-', line)[2].strip()
    return cli_flag, product_version, fun_err_msg


#need to explain this function.
def get_serial_number():
    """
    Function name      : getdev_type
    Function describe  : 获取主机SN号
    Input              : ipStr
    Return             : fun_flag, serialNumber, fun_err_msg
    """
    fun_flag = CHECK_FAIL
    fun_err_msg = ''
    serial_number = ''
    mac_str = ""
    mac_address = ''
    if_config_str = CLI.execCmdWithTimout("/sbin/ifconfig", 60)
    ip_link_show = CLI.execCmdWithTimout("/sbin/ip link show", 60)

    if ip_link_show.find("TOOLKIT_EXE_CMD_FAILED") > 0:
        ip_link_show = CLI.execCmdWithTimout("ip link show", 60)

    temp_list = if_config_str.splitlines()
    temp_list_2 = ip_link_show.splitlines()

    for i in range(0, len(temp_list)):
        if IP_ADDR in temp_list[i]:
            eth_name = temp_list[i - 1].split()[0]
            if eth_name.find(":") >= 0:
                eth_name = eth_name.split(':')[0]

            eth_name = eth_name.split(':')[0]
            for i in range(0, len(temp_list_2)):
                if eth_name in temp_list_2[i]:
                    mac_address = temp_list_2[i + 1].split()[1]
                    break
            mac_arr = mac_address.strip().split(':');

            for mac in mac_arr:
                if(len(mac) == 1):
                    mac_str = "0"
                else:
                    mac_str += mac
            if "0" == mac_str:
                serial_number = get_time_str() + IP_ADDR
            else:
                serial_number = mac_str + IP_ADDR
            fun_flag = CHECK_PASS
            break
    if '' == serial_number:
        serial_number = "SN" + get_time_str() + IP_ADDR
        if LANGUAGE == "zh":
            fun_err_msg = u"获取设备版本失败。"
        else:
            fun_err_msg = "Get product version failed."

    serial_number = serial_number.replace('.', '')

    return fun_flag, serial_number, fun_err_msg

def get_time_str():
    t = time.strftime('%Y-%m-%d-%H-%M-%S', time.localtime(time.time()))
    t = t.replace('-', '')
    return t
