﻿# -*- coding: UTF-8 -*-
import sys
import traceback
import re

class log():

    """
            功能：日志记录函数的子函数，用于获取调用函数和调用行号
            参数： MAX_CALLER_LEVEL：最大调用关系层数
            返回值：调用函数信息
    """
    @staticmethod
    def getCallerInfo(MAX_CALLER_LEVEL = 5, skipLastLevel = True):
        #从堆栈中获取调用函数和行号

        #初始化参数
        funcBack = sys._getframe().f_back
        if True == skipLastLevel: #忽略最近的调用关系
            funcBack = funcBack.f_back
            MAX_CALLER_LEVEL -= 1

        #生成函数调用关系
        callerInfo = ""
        for _ in range(0, MAX_CALLER_LEVEL):

            #获取该级调用函数和行号
            if hasattr(funcBack, "f_code") and hasattr(funcBack, "f_lineno"):
                funcName = funcBack.f_code.co_name
                lineNumber = funcBack.f_lineno
                callerInfo = " [" + str(funcName) + ":" + str(lineNumber) + "]" + callerInfo
            else:
                break

            #刷新Back函数
            if hasattr(funcBack, "f_back"):
                funcBack = funcBack.f_back
            else:
                break

        #返回函数调用关系
        return callerInfo

    @staticmethod
    def debug(context, info):
        """
                    功能：记录调试信息info到工具日志中
                    参数：context=工具上下文；info=要记录的信息
                    返回值：True=成功；False=失败
        """
        logInfo = info + log.getCallerInfo()
        if context and "Logger" in context:
            context.get("Logger").debug('[ToolLog]:' + logInfo)
            return True
        else:
            return False

    @staticmethod
    def error(context, info):
        """
                    功能：记录错误信息info到工具日志中
                    参数：context=工具上下文；info=要记录的信息
                    返回值：True=成功；False=失败
        """
        logInfo = info + log.getCallerInfo()
        if context and "Logger" in context:
            context.get("Logger").error('[ToolLog]:' + logInfo)
            return True
        else:
            return False

    @staticmethod
    def info(context, info):
        logInfo = info + log.getCallerInfo()
        if context and "Logger" in context:
            context.get("Logger").info('[ToolLog]:' + logInfo)
            return True
        else:
            return False

    @staticmethod
    def warn(context, info):
        logInfo = info + log.getCallerInfo()
        if context and "Logger" in context:
            context.get("Logger").warn('[ToolLog]:' + logInfo)
            return True
        else:
            return False
