# -*- coding: UTF-8 -*-
'''
netapp connector
'''
from netapp_connector import NetAppConnector
from netapp_exceptions import FailedParamException
MAJOR_VERSION = '1'
MINOR_VERSION = '3'

class NetAppProcessor(object):
    '''
    :docstring
    '''
    def __init__(self, connection_settings, LOGGER):
        '''
        :constructor
        '''
        self.logger = LOGGER
        self.host_addr = connection_settings.getConnectionIp()
        self.user_name = connection_settings.getUserName()
        self.passwd = connection_settings.getPasswd()
        self.port = connection_settings.getPort()
        self.logger.info("connect to netapp with port :" + str(self.port))
        self.server_type = connection_settings.getServerType()
        self.logger.info("connect to netapp server type  :" + str(self.server_type))
        self.connection_style = connection_settings.getConnectStyle()
        self.transport_type = connection_settings.getTransportType()
        self.logger.info("connect to netapp server with protocol:" + str(self.transport_type))
        self.netAppConnector = None
        self.prepare_connection()

    def prepare_connection(self):
        '''
        :param cmd:
        :return:
        '''
        self.netAppConnector = NetAppConnector(self.host_addr, MAJOR_VERSION, MINOR_VERSION)
        resultCode = self.netAppConnector.set_server_type(self.server_type)
        if resultCode and resultCode.results_errno() != 0:
            self.logger.error("connection failed : " + resultCode.results_reason())
            raise FailedParamException("failed setting server type")
        resultCode = self.netAppConnector.set_transport_type(self.transport_type)
        if resultCode and resultCode.results_errno() != 0:
            self.logger.error("connection failed : " + resultCode.results_reason())
            raise FailedParamException("failed setting transport type")
        resultCode = self.netAppConnector.set_port(self.port)
        if resultCode and resultCode.results_errno() != 0:
            self.logger.error("connection failed : " + resultCode.results_reason())
            raise FailedParamException("failed setting connection port.")
        resultCode = self.netAppConnector.set_login_type(self.connection_style)
        if resultCode and resultCode.results_errno() != 0:
            self.logger.error("connection failed : " + resultCode.results_reason())
            raise FailedParamException("failed setting filer login style")
        resultCode = self.netAppConnector.set_login_params(self.user_name, self.passwd)
        if resultCode and resultCode.results_errno() != 0:
            self.logger.error("connection failed : " + resultCode.results_reason())
            raise FailedParamException("authentication failed! current credential not work!")
        return

    def simply_invoke(self, cmd, *arg):
        '''
        :invoke ontap api
        :notice arg must not be odd, it shall be key - value params
        '''
        try:
            result = self.netAppConnector.request(cmd, *arg)
            result = result.to_pretty_xml()
            result = cmd + " " + " ".join(arg) + '\r\n' + result
            return result
        except BaseException, excp:
            self.logger.error("failed to invoke OnTap service: %s" % str(excp))
            raise
